# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from django.dispatch import Signal


user_created = Signal(providing_args=['user'])
user_created.__doc__ = """
Signal triggered when a user is automatically created during authentication.

:param sender:
    The function that created the user instance.

:param user:
    The user instance that was created.
"""



# This file mainly exists to allow python setup.py test to work.
# http://ericholscher.com/blog/2009/jun/29/enable-setuppy-test-your-django-apps/
import os
import sys

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'
os.environ['REUSE_DB'] = '0'

test_dir = os.path.join(os.path.dirname(__file__), 'django_browserid/tests')
sys.path.insert(0, test_dir)

import django

from django.test.utils import get_runner
from django.conf import settings
from django.core.management import call_command


def runtests():
    if django.VERSION >= (1, 7, 0):
        django.setup()
        call_command('migrate', interactive=False)

    else:
        call_command('syncdb', interactive=False)

    call_command('flush', interactive=False)
    test_runner = get_runner(settings)
    failures = test_runner(interactive=False, failfast=False).run_tests([])
    sys.exit(failures)

if __name__ == '__main__':
    runtests()



"""
django-browserid

This Source Code Form is subject to the terms of the Mozilla Public
License, v. 2.0. If a copy of the MPL was not distributed with this
file, You can obtain one at http://mozilla.org/MPL/2.0/.
"""
__version__ = '0.11'

from django_browserid.auth import (
    BrowserIDBackend,
    LocalBrowserIDBackend
)  # NOQA
from django_browserid.base import (
    BrowserIDException,
    get_audience,
    MockVerifier,
    RemoteVerifier,
    LocalVerifier,
    VerificationResult
)  # NOQA



from django.contrib import admin
from django.db import models

from mock import Mock

from django_browserid.admin import BrowserIDAdminSite
from django_browserid.tests import TestCase


class BrowserIDAdminSiteTests(TestCase):
    def test_copy_registry(self):
        """
        copy_registry should register the ModelAdmins from the given
        site on the BrowserIDAdminSite.
        """
        django_site = admin.AdminSite()
        browserid_site = BrowserIDAdminSite()

        class TestModel(models.Model):
            pass
        class TestModelAdmin(admin.ModelAdmin):
            pass

        browserid_site.register = Mock()
        django_site.register(TestModel, TestModelAdmin)

        browserid_site.copy_registry(django_site)
        browserid_site.register.assert_any_call(TestModel, TestModelAdmin)

    def test_copy_registry_multiple(self):
        django_site = admin.AdminSite()
        browserid_site = BrowserIDAdminSite()

        class TestModel1(models.Model):
            pass
        class TestModel2(models.Model):
            pass
        class TestModel3(models.Model):
            pass
        class TestModelAdmin(admin.ModelAdmin):
            pass
        class TestModel2Admin(admin.ModelAdmin):
            pass

        browserid_site.register = Mock()
        django_site.register(TestModel1, TestModelAdmin)
        django_site.register(TestModel2, TestModel2Admin)
        django_site.register(TestModel3, TestModelAdmin)

        browserid_site.copy_registry(django_site)
        browserid_site.register.assert_any_call(TestModel1, TestModelAdmin)
        browserid_site.register.assert_any_call(TestModel2, TestModel2Admin)
        browserid_site.register.assert_any_call(TestModel3, TestModelAdmin)



from django import template

from fancy_tag import fancy_tag

from django_browserid import helpers


register = template.Library()


@fancy_tag(register, takes_context=True)
def browserid_info(context, **kwargs):
    return helpers.browserid_info(**kwargs)


@fancy_tag(register, takes_context=True)
def browserid_login(context, **kwargs):
    return helpers.browserid_login(**kwargs)


@fancy_tag(register, takes_context=True)
def browserid_logout(context, **kwargs):
    return helpers.browserid_logout(**kwargs)


@fancy_tag(register, takes_context=True)
def browserid_js(context, **kwargs):
    return helpers.browserid_js(**kwargs)

@fancy_tag(register, takes_context=True)
def browserid_css(context, **kwargs):
    return helpers.browserid_css(**kwargs)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import logging

from django.conf.urls import patterns, url
from django.core.exceptions import ImproperlyConfigured

from django_browserid import views
from django_browserid.util import import_from_setting


logger = logging.getLogger(__name__)


try:
    Verify = import_from_setting('BROWSERID_VERIFY_CLASS')
    logger.debug('django_browserid using custom Verify view ' +
                 '.'.join([Verify.__module__, Verify.__name__]))
except ImproperlyConfigured as e:
    logger.debug('django_browserid using default Verify view.')
    Verify = views.Verify


urlpatterns = patterns('',
    url(r'^browserid/login/$', Verify.as_view(), name='browserid.login'),
    url(r'^browserid/logout/$', views.Logout.as_view(), name='browserid.logout'),
    url(r'^browserid/csrf/$', views.CsrfToken.as_view(), name='browserid.csrf'),
)



from django.core.urlresolvers import resolve
from django.test.client import RequestFactory
from django.utils.six.moves import reload_module

from mock import Mock

from django_browserid import urls
from django_browserid.views import Verify
from django_browserid.tests import TestCase


class MyVerifyClass(Verify):
    as_view = Mock()


class UrlTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_override_verify_class(self):
        """
        Reload so that the settings.BROWSERID_VERIFY_CLASS takes effect.
        """
        path = 'django_browserid.tests.test_urls.MyVerifyClass'
        with self.settings(BROWSERID_VERIFY_CLASS=path):
            reload_module(urls)

        view = resolve('/browserid/login/', urls).func
        self.assertEqual(view, MyVerifyClass.as_view.return_value)

        # Reset urls back to normal.
        reload_module(urls)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from django.conf.urls import include, patterns, url
from django.http import HttpResponse


urlpatterns = patterns('',
    (r'', include('django_browserid.urls')),
    url(r'^epic-fail/', lambda r: HttpResponse('this is a stub'),
        name='epic_fail')
)



import json

from django.core.exceptions import ImproperlyConfigured
from django.test.utils import override_settings
from django.utils import six
from django.utils.functional import lazy

from mock import Mock, patch
from nose.tools import eq_

from django_browserid.tests import TestCase
from django_browserid.util import import_from_setting, LazyEncoder


def _lazy_string():
    return 'blah'
lazy_string = lazy(_lazy_string, six.text_type)()


class TestLazyEncoder(TestCase):
    def test_lazy(self):
        thing = ['foo', lazy_string]
        thing_json = json.dumps(thing, cls=LazyEncoder)
        eq_('["foo", "blah"]', thing_json)


class ImportFromSettingTests(TestCase):
    def test_no_setting(self):
        """If the setting doesn't exist, raise ImproperlyConfigured."""
        with self.assertRaises(ImproperlyConfigured):
            import_from_setting('DOES_NOT_EXIST')

    @override_settings(TEST_SETTING={})
    def test_invalid_import(self):
        """
        If the setting isn't a proper string, raise
        ImproperlyConfigured.
        """
        with self.assertRaises(ImproperlyConfigured):
            import_from_setting('TEST_SETTING')

    @patch('django_browserid.util.import_module')
    @override_settings(TEST_SETTING='foo.bar.baz')
    def test_failed_import(self, import_module):
        """
        If there is an error importing the module, raise
        ImproperlyConfigured.
        """
        import_module.side_effect = ImportError
        with self.assertRaises(ImproperlyConfigured):
            import_from_setting('TEST_SETTING')
        import_module.assert_called_with('foo.bar')

    @patch('django_browserid.util.import_module')
    @override_settings(TEST_SETTING='foo.bar.baz')
    def test_error_importing(self, import_module):
        """
        If there is an error importing the module, raise
        ImproperlyConfigured.
        """
        import_module.side_effect = ImportError
        with self.assertRaises(ImproperlyConfigured):
            import_from_setting('TEST_SETTING')
        import_module.assert_called_with('foo.bar')

    @patch('django_browserid.util.import_module')
    @override_settings(TEST_SETTING='foo.bar.baz')
    def test_missing_attribute(self, import_module):
        """
        If the module is imported, but the function isn't found, raise
        ImproperlyConfigured.
        """
        import_module.return_value = Mock(spec=[])
        with self.assertRaises(ImproperlyConfigured):
            import_from_setting('TEST_SETTING')

    @patch('django_browserid.util.import_module')
    @override_settings(TEST_SETTING='foo.bar.baz')
    def test_existing_attribute(self, import_module):
        """
        If the module is imported and has the requested function,
        return it.
        """
        module = Mock(spec=['baz'])
        import_module.return_value = module
        self.assertEqual(import_from_setting('TEST_SETTING'), module.baz)




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import logging

from django.conf import settings
from django.contrib import auth
from django.http import HttpResponse
from django.template import RequestContext
from django.utils import six
from django.utils.http import is_safe_url
from django.views.decorators.cache import never_cache
from django.views.generic import View

from django_browserid.base import sanity_checks
from django_browserid.http import JSONResponse


logger = logging.getLogger(__name__)


class JSONView(View):
    def http_method_not_allowed(self, *args, **kwargs):
        response = JSONResponse({'error': 'Method not allowed.'}, status=405)
        allowed_methods = [m.upper() for m in self.http_method_names if hasattr(self, m)]
        response['Allow'] = ', '.join(allowed_methods)
        return response


def _get_next(request):
    """
    Get the next parameter from the request's POST arguments and
    validate it.

    :returns:
        The next parameter or None if it was not found or invalid.
    """
    next = request.POST.get('next')
    if is_safe_url(next, host=request.get_host()):
        return next
    else:
        return None


class Verify(JSONView):
    """
    Send an assertion to the remote verification service, and log the
    user in upon success.
    """
    @property
    def failure_url(self):
        """
        URL to redirect users to when login fails. This uses the value
        of ``settings.LOGIN_REDIRECT_URL_FAILURE``, and defaults to
        ``'/'`` if the setting doesn't exist.
        """
        return getattr(settings, 'LOGIN_REDIRECT_URL_FAILURE', '/')

    @property
    def success_url(self):
        """
        URL to redirect users to when login succeeds. This uses the
        value of ``settings.LOGIN_REDIRECT_URL``, and defaults to
        ``'/'`` if the setting doesn't exist.
        """
        return getattr(settings, 'LOGIN_REDIRECT_URL', '/')

    def login_success(self):
        """Log the user into the site."""
        auth.login(self.request, self.user)

        return JSONResponse({
            'email': self.user.email,
            'redirect': _get_next(self.request) or self.success_url
        })

    def login_failure(self):
        """
        Redirect the user to a login-failed page. By default a 403 is
        returned.
        """
        return JSONResponse({'redirect': self.failure_url}, status=403)

    def post(self, *args, **kwargs):
        """
        Send the given assertion to the remote verification service and,
        depending on the result, trigger login success or failure.
        """
        assertion = self.request.POST.get('assertion')
        if not assertion:
            return self.login_failure()

        self.user = auth.authenticate(request=self.request, assertion=assertion)
        if self.user and self.user.is_active:
            return self.login_success()

        return self.login_failure()

    def dispatch(self, request, *args, **kwargs):
        """
        Run some sanity checks on the request prior to dispatching it.
        """
        sanity_checks(request)
        return super(Verify, self).dispatch(request, *args, **kwargs)


class CsrfToken(JSONView):
    """Fetch a CSRF token for the frontend JavaScript."""
    @never_cache
    def get(self, request):
        # Different CSRF libraries (namely session_csrf) store the CSRF
        # token in different places. The only way to retrieve the token
        # that works with both the built-in CSRF and session_csrf is to
        # pull it from the template context processors via
        # RequestContext.
        context = RequestContext(request)

        # csrf_token might be a lazy value that triggers side-effects,
        # so we need to force it to a string.
        csrf_token = six.text_type(context.get('csrf_token', ''))

        return HttpResponse(csrf_token)


class Logout(JSONView):
    @property
    def redirect_url(self):
        """
        URL to redirect users to post-login. Uses
        ``settings.LOGOUT_REDIRECT_URL`` and defaults to ``/`` if the
        setting isn't found.
        """
        return getattr(settings, 'LOGOUT_REDIRECT_URL', '/')

    def post(self, request):
        """Log the user out."""
        auth.logout(request)

        return JSONResponse({
            'redirect': _get_next(self.request) or self.redirect_url
        })



from nose.tools import eq_

from django_browserid.http import JSONResponse
from django_browserid.tests import TestCase


class JSONResponseTests(TestCase):
    def test_basic(self):
        response = JSONResponse({'blah': 'foo', 'bar': 7})
        self.assert_json_equals(response.content, {'blah': 'foo', 'bar': 7})
        eq_(response.status_code, 200)

        response = JSONResponse(['baz', {'biff': False}])
        self.assert_json_equals(response.content, ['baz', {'biff': False}])
        eq_(response.status_code, 200)

    def test_status(self):
        response = JSONResponse({'blah': 'foo', 'bar': 7}, status=404)
        self.assert_json_equals(response.content, {'blah': 'foo', 'bar': 7})
        eq_(response.status_code, 404)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from django.contrib.admin.sites import AdminSite


class BrowserIDAdminSite(AdminSite):
    """Support logging in to the admin interface via BrowserID."""
    login_template = 'browserid/admin_login.html'

    #: If True, include the normal username and password form as well as
    #: the BrowserID button.
    include_password_form = False

    def copy_registry(self, site):
        """
        Copy the ModelAdmins that have been registered on another site
        so that they are available on this site as well.

        Useful when used with :func:`django.contrib.admin.autocomplete`,
        allowing you to copy the ModelAdmin entries registered with the
        default site, such as the User ModelAdmin. For example, in
        ``urls.py``:

        .. code-block:: python

           from django.contrib import admin
           admin.autodiscover()

           from django_browserid.admin import site as browserid_admin
           browserid_admin.copy_registry(admin.site)

           # To include: url(r'^admin/', include(browserid_admin.urls))

        :param site:
            Site to copy registry entries from.
        """
        for model, modeladmin in site._registry.items():
            self.register(model, modeladmin.__class__)

    def login(self, request, extra_context=None):
        # Add extra context variables to login view.
        extra_context = extra_context or {}
        extra_context['include_password_form'] = self.include_password_form
        return super(BrowserIDAdminSite, self).login(request, extra_context)


#: Global object for the common case. You can import this in
#: ``admin.py`` and ``urls.py`` instead of
#: :data:`django.contrib.admin.site`.
site = BrowserIDAdminSite()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import json

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.functional import Promise
from django.utils.importlib import import_module

try:
    from django.utils.encoding import force_unicode as force_text
except ImportError:
    from django.utils.encoding import force_text  # Python 3


class LazyEncoder(json.JSONEncoder):
    """
    JSONEncoder that turns Promises into unicode strings to support functions
    like ugettext_lazy and reverse_lazy.
    """
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return super(LazyEncoder, self).default(obj)


def import_from_setting(setting):
    """
    Attempt to load a module attribute from a module as specified by a setting.

    :raises:
        ImproperlyConfigured if anything goes wrong.
    """
    try:
        path = getattr(settings, setting)
    except AttributeError as e:
        raise ImproperlyConfigured('Setting {0} not found.'.format(setting))

    try:
        i = path.rfind('.')
        module, attr = path[:i], path[i + 1:]
    except AttributeError as e:
        raise ImproperlyConfigured('Setting {0} should be an import path.'.format(setting))

    try:
        mod = import_module(module)
    except ImportError as e:
        raise ImproperlyConfigured('Error importing `{0}`: {1}'.format(path, e))

    try:
        return getattr(mod, attr)
    except AttributeError as e:
        raise ImproperlyConfigured('Module {0} does not define `{1}`.'.format(module, attr))



from django.utils.functional import lazy

from mock import patch
from nose.tools import eq_

from django_browserid import helpers
from django_browserid.tests import TestCase


def _lazy_request_args():
    return {'siteName': 'asdf'}
lazy_request_args = lazy(_lazy_request_args, dict)


class BrowserIDInfoTests(TestCase):
    def setUp(self):
        patcher = patch('django_browserid.helpers.render_to_string')
        self.addCleanup(patcher.stop)
        self.render_to_string = patcher.start()

    def test_defaults(self):
        with self.settings(BROWSERID_REQUEST_ARGS={'foo': 'bar', 'baz': 1}):
            output = helpers.browserid_info()

        eq_(output, self.render_to_string.return_value)
        expected_info = {
            'loginUrl': '/browserid/login/',
            'logoutUrl': '/browserid/logout/',
            'csrfUrl': '/browserid/csrf/',
            'requestArgs': {'foo': 'bar', 'baz': 1},
        }
        self.render_to_string.assertCalledWith('browserid/info.html', {'info': expected_info})

    def test_lazy_request_args(self):
        with self.settings(BROWSERID_REQUEST_ARGS=lazy_request_args()):
            output = helpers.browserid_info()

        eq_(output, self.render_to_string.return_value)
        expected_info = {
            'loginUrl': '/browserid/login/',
            'logoutUrl': '/browserid/logout/',
            'csrfUrl': '/browserid/csrf/',
            'requestArgs': {'siteName': 'asdf'},
        }
        self.render_to_string.assertCalledWith('browserid/info.html', {'info': expected_info})


class BrowserIDJSTests(TestCase):
    def test_basic(self):
        output = helpers.browserid_js()
        self.assertHTMLEqual(output, """
            <script type="text/javascript" src="https://login.persona.org/include.js"></script>
            <script type="text/javascript" src="static/browserid/api.js"></script>
            <script type="text/javascript" src="static/browserid/browserid.js"></script>
        """)

    def test_no_shim(self):
        output = helpers.browserid_js(include_shim=False)
        self.assertHTMLEqual(output, """
            <script type="text/javascript" src="static/browserid/api.js"></script>
            <script type="text/javascript" src="static/browserid/browserid.js"></script>
        """)

    def test_custom_shim(self):
        with self.settings(BROWSERID_SHIM='http://example.com/test.js'):
            output = helpers.browserid_js()
        self.assertHTMLEqual(output, """
            <script type="text/javascript" src="http://example.com/test.js"></script>
            <script type="text/javascript" src="static/browserid/api.js"></script>
            <script type="text/javascript" src="static/browserid/browserid.js"></script>
        """)

    def test_autologin_email(self):
        """
        If BROWSERID_AUTOLOGIN_ENABLED is True, do not include the shim
        and include the autologin mock script.
        """
        with self.settings(BROWSERID_AUTOLOGIN_ENABLED=True):
            output = helpers.browserid_js()
            self.assertHTMLEqual(output, """
                <script type="text/javascript" src="static/browserid/api.js"></script>
                <script type="text/javascript" src="static/browserid/autologin.js"></script>
                <script type="text/javascript" src="static/browserid/browserid.js"></script>
            """)


class BrowserIDCSSTests(TestCase):
    def test_basic(self):
        output = helpers.browserid_css()
        self.assertHTMLEqual(output, """
            <link rel="stylesheet" href="static/browserid/persona-buttons.css" />
        """)


class BrowserIDButtonTests(TestCase):
    def test_basic(self):
        button = helpers.browserid_button(text='asdf', next='1234', link_class='fake-button',
                                          href="/test", attrs={'target': '_blank'})
        self.assertHTMLEqual(button, """
            <a href="/test" class="fake-button" data-next="1234" target="_blank">
                <span>asdf</span>
            </a>
        """)

    def test_json_attrs(self):
        button = helpers.browserid_button(text='qwer', next='5678', link_class='fake-button',
                                          attrs='{"target": "_blank"}')
        self.assertHTMLEqual(button, """
            <a href="#" class="fake-button" data-next="5678" target="_blank">
                <span>qwer</span>
            </a>
        """)


class BrowserIDLoginTests(TestCase):
    def test_login_class(self):
        with self.settings(LOGIN_REDIRECT_URL='/'):
            button = helpers.browserid_login(link_class='go button')
        self.assertHTMLEqual(button, """
            <a href="#" class="go button browserid-login" data-next="/">
                <span>Sign in</span>
            </a>
        """)

    def test_default_class(self):
        """
        If no class is provided, it should default to
        'browserid-login persona-button'.
        """
        with self.settings(LOGIN_REDIRECT_URL='/'):
            button = helpers.browserid_login()
        self.assertHTMLEqual(button, """
            <a href="#" class="browserid-login persona-button" data-next="/">
                <span>Sign in</span>
            </a>
        """)

    def test_color_class(self):
        with self.settings(LOGIN_REDIRECT_URL='/'):
            button = helpers.browserid_login(color='dark')
        self.assertHTMLEqual(button, """
            <a href="#" class="browserid-login persona-button dark" data-next="/">
                <span>Sign in</span>
            </a>
        """)

    def test_color_custom_class(self):
        """
        If using a color and a custom link class, persona-button should
        be added to the link class.
        """
        with self.settings(LOGIN_REDIRECT_URL='/'):
            button = helpers.browserid_login(link_class='go button', color='dark')
        self.assertHTMLEqual(button, """
            <a href="#" class="go button browserid-login persona-button dark" data-next="/">
                <span>Sign in</span>
            </a>
        """)

    def test_next(self):
        button = helpers.browserid_login(next='/foo/bar')
        self.assertHTMLEqual(button, """
            <a href="#" class="browserid-login persona-button" data-next="/foo/bar">
                <span>Sign in</span>
            </a>
        """)

    def test_next_default(self):
        """next should default to LOGIN_REDIRECT_URL"""
        with self.settings(LOGIN_REDIRECT_URL='/foo/bar'):
            button = helpers.browserid_login()
        self.assertHTMLEqual(button, """
            <a href="#" class="browserid-login persona-button" data-next="/foo/bar">
                <span>Sign in</span>
            </a>
        """)


class BrowserIDLogoutTests(TestCase):
    def test_logout_class(self):
        with self.settings(LOGOUT_REDIRECT_URL='/'):
            button = helpers.browserid_logout(link_class='go button')
        self.assertHTMLEqual(button, """
            <a href="/browserid/logout/" class="go button browserid-logout" data-next="/">
                <span>Sign out</span>
            </a>
        """)

    def test_next(self):
        button = helpers.browserid_logout(next='/foo/bar')
        self.assertHTMLEqual(button, """
            <a href="/browserid/logout/" class="browserid-logout" data-next="/foo/bar">
                <span>Sign out</span>
            </a>
        """)

    def test_next_default(self):
        """next should default to LOGOUT_REDIRECT_URL"""
        with self.settings(LOGOUT_REDIRECT_URL='/foo/bar'):
            button = helpers.browserid_logout()
        self.assertHTMLEqual(button, """
            <a href="/browserid/logout/" class="browserid-logout" data-next="/foo/bar">
                <span>Sign out</span>
            </a>
        """)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import json

from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.utils.six import string_types

from django_browserid.compat import jingo_register, reverse
from django_browserid.util import LazyEncoder

MANDATORY_LINK_CLASS_LOGIN = 'browserid-login'
DEFAULT_LINK_CLASS_LOGIN = MANDATORY_LINK_CLASS_LOGIN + ' persona-button'
MANDATORY_LINK_CLASS_LOGOUT = 'browserid-logout'
DEFAULT_LINK_CLASS_LOGOUT = MANDATORY_LINK_CLASS_LOGOUT


@jingo_register.function
def browserid_info():
    """
    Output the HTML for the info tag, which contains the arguments for
    navigator.id.request from the BROWSERID_REQUEST_ARGS setting. Should
    be called once at the top of the page just below the <body> tag.
    """
    # Force request_args to be a dictionary, in case it is lazily generated.
    request_args = dict(getattr(settings, 'BROWSERID_REQUEST_ARGS', {}))

    info = json.dumps({
        'loginUrl': reverse('browserid.login'),
        'logoutUrl': reverse('browserid.logout'),
        'csrfUrl': reverse('browserid.csrf'),
        'requestArgs': request_args,
    }, cls=LazyEncoder)

    return render_to_string('browserid/info.html', {
        'info': info,
    })


def browserid_button(text=None, next=None, link_class=None, attrs=None, href='#'):
    """
    Output the HTML for a BrowserID link.

    :param text:
        Text to use inside the link.

    :param next:
        Value to use for the data-next attribute on the link.

    :param link_class:
        Class to use for the link.

    :param attrs:
        Dictionary of attributes to add to the link. Values here override those
        set by other arguments.

        If given a string, it is parsed as JSON and is expected to be an object.

    :param href:
        href to use for the link.
    """
    attrs = attrs or {}
    if isinstance(attrs, string_types):
        attrs = json.loads(attrs)

    attrs.setdefault('class', link_class)
    attrs.setdefault('href', href)
    attrs.setdefault('data-next', next)
    return render_to_string('browserid/button.html', {
        'text': text,
        'attrs': attrs,
    })


@jingo_register.function
def browserid_login(text='Sign in', color=None, next=None,
                    link_class=DEFAULT_LINK_CLASS_LOGIN,
                    attrs=None, fallback_href='#'):
    """
    Output the HTML for a BrowserID login link.

    :param text:
        Text to use inside the link. Defaults to 'Sign in', which is not
        localized.

    :param color:
        Color to use for the login button; this will only work if you have
        included the default CSS provided by
        :py:func:`django_browserid.helpers.browserid_css`.

        Supported colors are: `'dark'`, `'blue'`, and `'orange'`.

    :param next:
        URL to redirect users to after they login from this link. If omitted,
        the LOGIN_REDIRECT_URL setting will be used.

    :param link_class:
        CSS class for the link. Defaults to `browserid-login persona-button`.

    :param attrs:
        Dictionary of attributes to add to the link. Values here override those
        set by other arguments.

        If given a string, it is parsed as JSON and is expected to be an object.

    :param fallback_href:
        Value to use for the href of the link. If the user has disabled
        JavaScript, the login link will bring them to this page, which can be
        used as a non-JavaScript login fallback.
    """
    if MANDATORY_LINK_CLASS_LOGIN not in link_class.split(' '):
        link_class += ' ' + MANDATORY_LINK_CLASS_LOGIN

    if color:
        if 'persona-button' not in link_class.split(' '):
            link_class += ' persona-button {0}'.format(color)
        else:
            link_class += ' ' + color

    next = next or getattr(settings, 'LOGIN_REDIRECT_URL', '/')
    return browserid_button(text, next, link_class, attrs, fallback_href)


@jingo_register.function
def browserid_logout(text='Sign out', next=None,
                     link_class=DEFAULT_LINK_CLASS_LOGOUT, attrs=None):
    """
    Output the HTML for a BrowserID logout link.

    :param text:
        Text to use inside the link. Defaults to 'Sign out', which is not
        localized.

    :param link_class:
        CSS classes for the link. The classes will be appended to the
        default class `browserid-logout`.

    :param attrs:
        Dictionary of attributes to add to the link. Values here override those
        set by other arguments.

        If given a string, it is parsed as JSON and is expected to be an object.
    """
    next = next or getattr(settings, 'LOGOUT_REDIRECT_URL', '/')

    if MANDATORY_LINK_CLASS_LOGOUT not in link_class.split(' '):
        link_class += ' ' + MANDATORY_LINK_CLASS_LOGOUT

    return browserid_button(text, next, link_class,
                            attrs, reverse('browserid.logout'))


@jingo_register.function
def browserid_js(include_shim=True):
    """
    Return <script> tags for the JavaScript required by the BrowserID login
    button. Requires use of the staticfiles app.

    If the BROWSERID_AUTOLOGIN_ENABLED setting is True, an extra JavaScript
    file for mocking out Persona will be included, and the shim won't
    be included regardless of the value of the ``include_shim`` setting.

    :param include_shim:
        A boolean that determines if the persona.org JavaScript shim is included
        in the output. Useful if you want to minify the button JavaScript using
        a library like django-compressor that can't handle external JavaScript.
    """
    files = []
    autologin_enabled = getattr(settings, 'BROWSERID_AUTOLOGIN_ENABLED', False)

    # Include navigator.id shim only if we're not doing autologin.
    if include_shim and not autologin_enabled:
        files.append(getattr(settings, 'BROWSERID_SHIM', 'https://login.persona.org/include.js'))

    # Include django-browserid API
    files.append(staticfiles_storage.url('browserid/api.js'))

    # If we're doing autologin, include the JS to mock out certain parts
    # of the API.
    if autologin_enabled:
        files.append(staticfiles_storage.url('browserid/autologin.js'))

    # Include the JS to bind to login buttons.
    files.append(staticfiles_storage.url('browserid/browserid.js'))

    tags = ['<script type="text/javascript" src="{0}"></script>'.format(path)
            for path in files]
    return mark_safe('\n'.join(tags))


@jingo_register.function
def browserid_css():
    """
    Return <link> tag for the optional CSS included with django-browserid.
    Requires use of the staticfiles app.
    """
    url = staticfiles_storage.url('browserid/persona-buttons.css')
    return mark_safe('<link rel="stylesheet" href="{0}" />'.format(url))



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import logging
from datetime import datetime

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils import six
from django.utils.encoding import python_2_unicode_compatible
from django.utils.http import same_origin

import requests

from django_browserid.compat import pybrowserid_found


logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class BrowserIDException(Exception):
    """Raised when there is an issue verifying an assertion."""
    def __init__(self, exc):
        #: Original exception that caused this to be raised.
        self.exc = exc

    def __str__(self):
        return six.text_type(self.exc)


def sanity_checks(request):
    """
    Small checks for common errors.

    Checks are normally only enabled if DEBUG is True. You can
    explicitly disable the checks using the
    BROWSERID_DISABLE_SANITY_CHECKS.

    :returns:
        True if the checks were run, False if they were skipped.
    """
    if getattr(settings, 'BROWSERID_DISABLE_SANITY_CHECKS', not settings.DEBUG):
        return False  # Return value helps us test if the checks ran.

    # SESSION_COOKIE_SECURE should be False in development unless you can
    # use https.
    if settings.SESSION_COOKIE_SECURE and not request.is_secure():
        logger.warning('SESSION_COOKIE_SECURE is currently set to True, '
                       'which may cause issues with django_browserid '
                       'login during local development. Consider setting '
                       'it to False.')

    # If you're using django-csp, you should include persona.
    if 'csp.middleware.CSPMiddleware' in settings.MIDDLEWARE_CLASSES:
        persona = 'https://login.persona.org'
        in_default = persona in getattr(settings, 'CSP_DEFAULT_SRC', ())
        in_script = persona in getattr(settings, 'CSP_SCRIPT_SRC', ())
        in_frame = persona in getattr(settings, 'CSP_FRAME_SRC', ())

        if (not in_script or not in_frame) and not in_default:
            logger.warning('django-csp detected, but %s was not found in '
                           'your CSP policies. Consider adding it to '
                           'CSP_SCRIPT_SRC and CSP_FRAME_SRC',
                           persona)

    return True


def get_audience(request):
    """
    Determine the audience to use for verification from the given request.

    Relies on the BROWSERID_AUDIENCES setting, which is an explicit list of acceptable
    audiences for your site.

    :returns:
        The first audience in BROWSERID_AUDIENCES that has the same origin as the request's
        URL.

    :raises:
        :class:`django.core.exceptions.ImproperlyConfigured`: If BROWSERID_AUDIENCES isn't
        defined, or if no matching audience could be found.
    """
    protocol = 'https' if request.is_secure() else 'http'
    host = '{0}://{1}'.format(protocol, request.get_host())
    try:
        audiences = settings.BROWSERID_AUDIENCES
        if not audiences and settings.DEBUG:
            return host
    except AttributeError:
        if settings.DEBUG:
            return host
        raise ImproperlyConfigured('Required setting BROWSERID_AUDIENCES not found!')

    for audience in audiences:
        if same_origin(host, audience):
            return audience

    # No audience found? We must not be configured properly, otherwise why are we getting this
    # request?
    raise ImproperlyConfigured('No audience could be found in BROWSERID_AUDIENCES for host `{0}`.'
                               .format(host))


@python_2_unicode_compatible
class VerificationResult(object):
    """
    Result of an attempt to verify an assertion.

    VerificationResult objects can be treated as booleans to test if the verification succeeded or
    not.

    The fields returned by the remote verification service, such as ``email`` or ``issuer``, are
    available as attributes if they were included in the response. For example, a failure result
    will raise an AttributeError if you try to access the ``email`` attribute.
    """
    def __init__(self, response):
        """
        :param response:
            Dictionary of the response from the remote verification service.
        """
        self._response = response

    def __getattr__(self, name):
        if name in self._response:
            return self._response[name]
        else:
            raise AttributeError

    @property
    def expires(self):
        """The expiration date of the assertion as a naive :class:`datetime.datetime` in UTC."""
        try:
            return datetime.utcfromtimestamp(int(self._response['expires']) / 1000.0)
        except KeyError:
            raise AttributeError
        except ValueError:
            timestamp = self._response['expires']
            logger.warning('Could not parse expires timestamp: `{0}`'.format(timestamp))
            return timestamp

    def __nonzero__(self):
        return self._response.get('status') == 'okay'

    def __bool__(self):
        return self.__nonzero__()

    def __str__(self):
        result = six.u('Success') if self else six.u('Failure')
        email = getattr(self, 'email', None)
        email_string = six.u(' email={0}').format(email) if email else six.u('')
        return six.u('<VerificationResult {0}{1}>').format(result, email_string)


class RemoteVerifier(object):
    """
    Verifies BrowserID assertions using a remote verification service.

    By default, this uses the Mozilla Persona service for remote verification.
    """
    verification_service_url = 'https://verifier.login.persona.org/verify'
    requests_parameters = {
        'timeout': 5
    }

    def verify(self, assertion, audience, **kwargs):
        """
        Verify an assertion using a remote verification service.

        :param assertion:
            BrowserID assertion to verify.

        :param audience:
            The protocol, hostname and port of your website. Used to confirm that the assertion was
            meant for your site and not for another site.

        :param kwargs:
            Extra keyword arguments are passed on to requests.post to allow customization.

        :returns:
            :class:`.VerificationResult`

        :raises:
            :class:`.BrowserIDException`: Error connecting to the remote verification service, or
            error parsing the response received from the service.
        """
        parameters = dict(self.requests_parameters, **{
            'data': {
                'assertion': assertion,
                'audience': audience,
            }
        })
        parameters['data'].update(kwargs)

        try:
            response = requests.post(self.verification_service_url, **parameters)
        except requests.exceptions.RequestException as err:
            raise BrowserIDException(err)

        try:
            return VerificationResult(response.json())
        except (ValueError, TypeError) as err:
            # If the returned JSON is invalid, log a warning and return a failure result.
            logger.warning('Failed to parse remote verifier response: `{0}`'
                           .format(response.content))
            return VerificationResult({
                'status': 'failure',
                'reason': 'Could not parse verifier response: {0}'.format(err)
            })


class MockVerifier(object):
    """Mock-verifies BrowserID assertions."""

    def __init__(self, email, **kwargs):
        """
        :param email:
            Email address to include in successful verification result. If None, verify will return
            a failure result.

        :param kwargs:
            Extra keyword arguments are used to update successful verification results. This allows
            for mocking attributes on the result, such as the issuer.
        """
        self.email = email
        self.result_attributes = kwargs

    def verify(self, assertion, audience, **kwargs):
        """
        Mock-verify an assertion. The return value is determined by the parameters given to the
        constructor.
        """
        if not self.email:
            return VerificationResult({
                'status': 'failure',
                'reason': 'No email given to MockVerifier.'
            })
        else:
            result = {
                'status': 'okay',
                'audience': audience,
                'email': self.email,
                'issuer': 'mockissuer.example.com:443',
                'expires': '1311377222765'
            }
            result.update(self.result_attributes)
            return VerificationResult(result)


if pybrowserid_found:
    from browserid.errors import Error as PyBrowserIDError
    from browserid.verifiers.local import LocalVerifier as PyBrowserIDLocalVerifier

    class LocalVerifier(object):
        """
        Verifies BrowserID assertions locally instead of using the remote
        verification service.
        """
        def __init__(self, *args, **kwargs):
            super(LocalVerifier, self).__init__(*args, **kwargs)
            self.pybid_verifier = PyBrowserIDLocalVerifier()

        def verify(self, assertion, audience, **kwargs):
            """
            Verify an assertion locally.

            :param assertion:
                BrowserID assertion to verify.

            :param audience:
                The protocol, hostname and port of your website. Used to confirm that the assertion was
                meant for your site and not for another site.

            :returns:
                :class:`.VerificationResult`
            """
            try:
                result = self.pybid_verifier.verify(assertion, audience)
            except PyBrowserIDError as error:
                return VerificationResult({
                    'status': 'failure',
                    'reason': error
                })

            return VerificationResult(result)
else:
    # If someone tries to use LocalVerifier, let's show a helpful error
    # instead of just raising an ImportError.
    class LocalVerifier(object):
        def __init__(self, *args, **kwargs):
            raise RuntimeError('You\'re attempting to use local assertion verification without '
                               'PyBrowserID installed. Please install PyBrowserID in order to '
                               'enable local assertion verification.')



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from datetime import datetime

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.utils import six

import requests
from mock import Mock, patch
from nose.plugins.skip import SkipTest
from nose.tools import eq_, ok_

from django_browserid import base
from django_browserid.compat import pybrowserid_found
from django_browserid.tests import TestCase


class SanityCheckTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    @override_settings(DEBUG=True)
    def test_debug_true(self):
        """
        If DEBUG is True and BROWSERID_DISABLE_SANITY_CHECKS is not set,
        run the checks.
        """
        request = self.factory.get('/')
        ok_(base.sanity_checks(request))

    @override_settings(DEBUG=False)
    def test_debug_false(self):
        """
        If DEBUG is True and BROWSERID_DISABLE_SANITY_CHECKS is not set,
        run the checks.
        """
        request = self.factory.get('/')
        ok_(not base.sanity_checks(request))

    @override_settings(BROWSERID_DISABLE_SANITY_CHECKS=True)
    def test_disable_sanity_checks(self):
        """
        If BROWSERID_DISABLE_SANITY_CHECKS is True, do not run any
        checks.
        """
        request = self.factory.get('/')
        ok_(not base.sanity_checks(request))

    @override_settings(BROWSERID_DISABLE_SANITY_CHECKS=False, SESSION_COOKIE_SECURE=True)
    def test_sanity_session_cookie(self):
        """
        If SESSION_COOKIE_SECURE == True and the current request isn't
        https, log a debug message warning about it.
        """
        request = self.factory.get('/')
        request.is_secure = Mock(return_value=False)
        with patch('django_browserid.base.logger.warning') as warning:
            base.sanity_checks(request)
        ok_(warning.called)

    @override_settings(BROWSERID_DISABLE_SANITY_CHECKS=False,
                       MIDDLEWARE_CLASSES=['csp.middleware.CSPMiddleware'])
    @patch('django_browserid.base.logger.warning')
    def test_sanity_csp(self, warning):
        """
        If the django-csp middleware is present and Persona isn't
        allowed by CSP, log a debug message warning about it.
        """
        request = self.factory.get('/')

        # Test if allowed properly.
        with self.settings(CSP_DEFAULT_SRC=[],
                           CSP_SCRIPT_SRC=['https://login.persona.org'],
                           CSP_FRAME_SRC=['https://login.persona.org']):
            base.sanity_checks(request)
        ok_(not warning.called)
        warning.reset_mock()

        # Test fallback to default-src.
        with self.settings(CSP_DEFAULT_SRC=['https://login.persona.org'],
                           CSP_SCRIPT_SRC=[],
                           CSP_FRAME_SRC=[]):
            base.sanity_checks(request)
        ok_(not warning.called)
        warning.reset_mock()

        # Test incorrect csp.
        with self.settings(CSP_DEFAULT_SRC=[],
                           CSP_SCRIPT_SRC=[],
                           CSP_FRAME_SRC=[]):
            base.sanity_checks(request)
        ok_(warning.called)
        warning.reset_mock()

        # Test partial incorrectness.
        with self.settings(CSP_DEFAULT_SRC=[],
                           CSP_SCRIPT_SRC=['https://login.persona.org'],
                           CSP_FRAME_SRC=[]):
            base.sanity_checks(request)
        ok_(warning.called)

    @override_settings(BROWSERID_DISABLE_SANITY_CHECKS=False,
                       MIDDLEWARE_CLASSES=['csp.middleware.CSPMiddleware'])
    @patch('django_browserid.base.logger.warning')
    def test_unset_csp(self, warning):
        """Check for errors when CSP settings aren't specified."""
        request = self.factory.get('/')
        correct = ['https://login.persona.org']
        setting_kwargs = {
            'CSP_DEFAULT_SRC': correct,
            'CSP_SCRIPT_SRC': correct,
            'CSP_FRAME_SRC': correct
        }

        # There's no easy way to use a variable for deleting the
        # attribute on the settings object, so we can't easily turn this
        # into a function, sadly.
        with self.settings(**setting_kwargs):
            del settings.CSP_DEFAULT_SRC
            base.sanity_checks(request)
        ok_(not warning.called)
        warning.reset_mock()

        with self.settings(**setting_kwargs):
            del settings.CSP_FRAME_SRC
            base.sanity_checks(request)
        ok_(not warning.called)
        warning.reset_mock()

        with self.settings(**setting_kwargs):
            del settings.CSP_SCRIPT_SRC
            base.sanity_checks(request)
        ok_(not warning.called)
        warning.reset_mock()


class GetAudienceTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_setting_missing(self):
        """
        If BROWSERID_AUDIENCES isn't defined, raise
        ImproperlyConfigured.
        """
        request = self.factory.get('/')

        with patch('django_browserid.base.settings') as settings:
            del settings.BROWSERID_AUDIENCES
            settings.DEBUG = False

            with self.assertRaises(ImproperlyConfigured):
                base.get_audience(request)

    def test_same_origin_found(self):
        """
        If an audience is found in BROWSERID_AUDIENCES with the same
        origin as the request URI, return it.
        """
        request = self.factory.get('http://testserver')

        audiences = ['https://example.com', 'http://testserver']
        with self.settings(BROWSERID_AUDIENCES=audiences, DEBUG=False):
            eq_(base.get_audience(request), 'http://testserver')

    def test_no_audience(self):
        """
        If no matching audiences is found in BROWSERID_AUDIENCES, raise
        ImproperlyConfigured.
        """
        request = self.factory.get('http://testserver')

        with self.settings(BROWSERID_AUDIENCES=['https://example.com']):
            with self.assertRaises(ImproperlyConfigured):
                base.get_audience(request)

    def test_missing_setting_but_in_debug(self):
        """
        If no BROWSERID_AUDIENCES is set but in DEBUG just use the
        current protocal and host.
        """
        request = self.factory.get('/')

        # Simulate that no BROWSERID_AUDIENCES has been set
        with patch('django_browserid.base.settings') as settings:
            del settings.BROWSERID_AUDIENCES
            settings.DEBUG = True
            eq_(base.get_audience(request), 'http://testserver')

    def test_no_audience_but_in_debug(self):
        """
        If no BROWSERID_AUDIENCES is set but in DEBUG just use the
        current protocal and host.
        """
        request = self.factory.get('/')

        # Simulate that no BROWSERID_AUDIENCES has been set
        with self.settings(BROWSERID_AUDIENCES=[], DEBUG=True):
            eq_(base.get_audience(request), 'http://testserver')


class VerificationResultTests(TestCase):
    def test_getattr_attribute_exists(self):
        """
        If a value exists in the response dict, it should be an
        attribute on the result.
        """
        result = base.VerificationResult({'myattr': 'foo'})
        eq_(result.myattr, 'foo')

    def test_getattr_attribute_doesnt_exist(self):
        """
        If a value doesn't exist in the response dict, accessing it as
        an attribute should raise an AttributeError.
        """
        result = base.VerificationResult({'myattr': 'foo'})
        with self.assertRaises(AttributeError):
            result.bar

    def test_expires_no_attribute(self):
        """
        If no expires attribute was in the response, raise an
        AttributeError.
        """
        result = base.VerificationResult({'myattr': 'foo'})
        with self.assertRaises(AttributeError):
            result.expires

    def test_expires_invalid_timestamp(self):
        """
        If the expires attribute cannot be parsed as a timestamp, return
        the raw string instead.
        """
        result = base.VerificationResult({'expires': 'foasdfhas'})
        eq_(result.expires, 'foasdfhas')

    def test_expires_valid_timestamp(self):
        """
        If expires contains a valid millisecond timestamp, return a
        corresponding datetime.
        """
        result = base.VerificationResult({'expires': '1379307128000'})
        eq_(datetime(2013, 9, 16, 4, 52, 8), result.expires)

    def test_nonzero_failure(self):
        """
        If the response status is not 'okay', the result should be
        falsy.
        """
        ok_(not base.VerificationResult({'status': 'failure'}))

    def test_nonzero_okay(self):
        """
        If the response status is 'okay', the result should be truthy.
        """
        ok_(base.VerificationResult({'status': 'okay'}))

    def test_str_success(self):
        """
        If the result is successful, include 'Success' and the email in
        the string.
        """
        result = base.VerificationResult({'status': 'okay', 'email': 'a@example.com'})
        eq_(six.text_type(result), '<VerificationResult Success email=a@example.com>')

        # If the email is missing, don't include it.
        result = base.VerificationResult({'status': 'okay'})
        eq_(six.text_type(result), '<VerificationResult Success>')

    def test_str_failure(self):
        """
        If the result is a failure, include 'Failure' in the string.
        """
        result = base.VerificationResult({'status': 'failure'})
        eq_(six.text_type(result), '<VerificationResult Failure>')

    def test_str_unicode(self):
        """Ensure that __str__ can handle unicode values."""
        result = base.VerificationResult({'status': 'okay', 'email': six.u('\x80@example.com')})
        eq_(six.text_type(result), six.u('<VerificationResult Success email=\x80@example.com>'))


class RemoteVerifierTests(TestCase):
    def _response(self, **kwargs):
        return Mock(spec=requests.Response, **kwargs)

    def test_verify_requests_parameters(self):
        """
        If a subclass overrides requests_parameters, the parameters
        should be passed to requests.post.
        """
        class MyVerifier(base.RemoteVerifier):
            requests_parameters = {'foo': 'bar'}
        verifier = MyVerifier()

        with patch('django_browserid.base.requests.post') as post:
            post.return_value = self._response(content='{"status":"failure"}')
            verifier.verify('asdf', 'http://testserver')

        # foo parameter passed with 'bar' value.
        eq_(post.call_args[1]['foo'], 'bar')

    def test_verify_kwargs(self):
        """
        Any keyword arguments passed to verify should be passed on as
        POST arguments.
        """
        verifier = base.RemoteVerifier()

        with patch('django_browserid.base.requests.post') as post:
            post.return_value = self._response(content='{"status":"failure"}')
            verifier.verify('asdf', 'http://testserver', foo='bar', baz=5)

        # foo parameter passed with 'bar' value.
        eq_(post.call_args[1]['data']['foo'], 'bar')
        eq_(post.call_args[1]['data']['baz'], 5)

    def test_verify_request_exception(self):
        """
        If a RequestException is raised during the POST, raise a
        BrowserIDException with the RequestException as the cause.
        """
        verifier = base.RemoteVerifier()
        request_exception = requests.exceptions.RequestException()

        with patch('django_browserid.base.requests.post') as post:
            post.side_effect = request_exception
            with self.assertRaises(base.BrowserIDException) as cm:
                verifier.verify('asdf', 'http://testserver')

        eq_(cm.exception.exc, request_exception)

    def test_verify_invalid_json(self):
        """
        If the response contains invalid JSON, return a failure result.
        """
        verifier = base.RemoteVerifier()

        with patch('django_browserid.base.requests.post') as post:
            response = self._response(content='{asg9=3{{{}}{')
            response.json.side_effect = ValueError("Couldn't parse json")
            post.return_value = response
            result = verifier.verify('asdf', 'http://testserver')
        ok_(not result)
        ok_(result.reason.startswith('Could not parse verifier response'))


    def test_verify_success(self):
        """
        If the response contains valid JSON, return a result object for
        that response.
        """
        verifier = base.RemoteVerifier()

        with patch('django_browserid.base.requests.post') as post:
            response = self._response(
                content='{"status": "okay", "email": "foo@example.com"}')
            response.json.return_value = {"status": "okay", "email": "foo@example.com"}
            post.return_value = response
            result = verifier.verify('asdf', 'http://testserver')
        ok_(result)
        eq_(result.email, 'foo@example.com')


class MockVerifierTests(TestCase):
    def test_verify_no_email(self):
        """
        If the given email is None, verify should return a failure
        result.
        """
        verifier = base.MockVerifier(None)
        result = verifier.verify('asdf', 'http://testserver')
        ok_(not result)
        eq_(result.reason, 'No email given to MockVerifier.')

    def test_verify_email(self):
        """
        If an email is given to the constructor, return a successful
        result.
        """
        verifier = base.MockVerifier('a@example.com')
        result = verifier.verify('asdf', 'http://testserver')
        ok_(result)
        eq_(result.audience, 'http://testserver')
        eq_(result.email, 'a@example.com')

    def test_verify_result_attributes(self):
        """Extra kwargs to the constructor are added to the result."""
        verifier = base.MockVerifier('a@example.com', foo='bar', baz=5)
        result = verifier.verify('asdf', 'http://testserver')
        eq_(result.foo, 'bar')
        eq_(result.baz, 5)


class LocalVerifierTests(TestCase):
    def setUp(self):
        # Skip tests if PyBrowserID is not installed.
        if not pybrowserid_found:
            raise SkipTest

        self.verifier = base.LocalVerifier()

    def test_verify_error(self):
        """
        If verify raises a PyBrowserIDError, return a failure
        result.
        """
        from browserid.errors import Error as PyBrowserIDError

        pybid_verifier = Mock()
        error = PyBrowserIDError()
        self.verifier.pybid_verifier = pybid_verifier

        pybid_verifier.verify.side_effect = error

        result = self.verifier.verify('asdf', 'qwer')
        pybid_verifier.verify.assert_called_with('asdf', 'qwer')
        self.assertFalse(result)
        self.assertEqual(result.reason, error)

    def test_verify_success(self):
        pybid_verifier = Mock()
        self.verifier.pybid_verifier = pybid_verifier

        response = {'status': 'okay'}
        pybid_verifier.verify.return_value = response

        result = self.verifier.verify('asdf', 'qwer')
        pybid_verifier.verify.assert_called_with('asdf', 'qwer')
        self.assertTrue(result)
        self.assertEqual(result._response, response)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
TEST_RUNNER = 'django_nose.runner.NoseTestSuiteRunner'

SECRET_KEY = 'asdf'

DATABASES = {
    'default': {
        'NAME': 'test.db',
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

INSTALLED_APPS = (
    'django_nose',
    'django_browserid',
    'django_browserid.tests',

    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.staticfiles',
)

ROOT_URLCONF = 'django_browserid.tests.urls'

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    'django_browserid.auth.BrowserIDBackend',
)

BROWSERID_CREATE_USER = True
BROWSERID_USERNAME_ALGO = None

STATIC_URL = 'static/'

BROWSERID_AUDIENCES = ['http://testserver']



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from django.contrib import auth
from django.test.client import RequestFactory
from django.utils import six
from django.utils.functional import lazy

from mock import Mock, patch, PropertyMock
from nose.tools import eq_, ok_

from django_browserid import BrowserIDException, views
from django_browserid.tests import mock_browserid, TestCase


class JSONViewTests(TestCase):
    def test_http_method_not_allowed(self):
        class TestView(views.JSONView):
            def get(self, request, *args, **kwargs):
                return 'asdf'
        response = TestView().http_method_not_allowed()
        eq_(response.status_code, 405)
        ok_(set(['GET']).issubset(set(response['Allow'].split(', '))))
        self.assert_json_equals(response.content, {'error': 'Method not allowed.'})

    def test_http_method_not_allowed_allowed_methods(self):
        class GetPostView(views.JSONView):
            def get(self, request, *args, **kwargs):
                return 'asdf'

            def post(self, request, *args, **kwargs):
                return 'qwer'
        response = GetPostView().http_method_not_allowed()
        ok_(set(['GET', 'POST']).issubset(set(response['Allow'].split(', '))))

        class GetPostPutDeleteHeadView(views.JSONView):
            def get(self, request, *args, **kwargs):
                return 'asdf'

            def post(self, request, *args, **kwargs):
                return 'qwer'

            def put(self, request, *args, **kwargs):
                return 'qwer'

            def delete(self, request, *args, **kwargs):
                return 'qwer'

            def head(self, request, *args, **kwargs):
                return 'qwer'
        response = GetPostPutDeleteHeadView().http_method_not_allowed()
        expected_methods = set(['GET', 'POST', 'PUT', 'DELETE', 'HEAD'])
        actual_methods = set(response['Allow'].split(', '))
        ok_(expected_methods.issubset(actual_methods))


class GetNextTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_no_param(self):
        """If next isn't in the POST params, return None."""
        request = self.factory.post('/')
        eq_(views._get_next(request), None)

    def test_is_safe(self):
        """Return the value of next if it is considered safe."""
        request = self.factory.post('/', {'next': '/asdf'})
        request.get_host = lambda: 'myhost'

        with patch.object(views, 'is_safe_url', return_value=True) as is_safe_url:
            eq_(views._get_next(request), '/asdf')
            is_safe_url.assert_called_with('/asdf', host='myhost')

    def test_isnt_safe(self):
        """If next isn't safe, return None."""
        request = self.factory.post('/', {'next': '/asdf'})
        request.get_host = lambda: 'myhost'

        with patch.object(views, 'is_safe_url', return_value=False) as is_safe_url:
            eq_(views._get_next(request), None)
            is_safe_url.assert_called_with('/asdf', host='myhost')


class VerifyTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def verify(self, request_type, **kwargs):
        """
        Call the verify view function. Kwargs are passed as GET or POST
        arguments.
        """
        if request_type == 'get':
            request = self.factory.get('/browserid/verify', kwargs)
        else:
            request = self.factory.post('/browserid/verify', kwargs)

        verify_view = views.Verify.as_view()
        with patch.object(auth, 'login'):
            response = verify_view(request)

        return response

    def test_no_assertion(self):
        """If no assertion is given, return a failure result."""
        with self.settings(LOGIN_REDIRECT_URL_FAILURE='/fail'):
            response = self.verify('post', blah='asdf')
        eq_(response.status_code, 403)
        self.assert_json_equals(response.content, {'redirect': '/fail'})

    @mock_browserid(None)
    def test_auth_fail(self):
        """If authentication fails, redirect to the failure URL."""
        with self.settings(LOGIN_REDIRECT_URL_FAILURE='/fail'):
            response = self.verify('post', assertion='asdf')
        eq_(response.status_code, 403)
        self.assert_json_equals(response.content, {'redirect': '/fail'})

    @mock_browserid('test@example.com')
    def test_auth_success_redirect_success(self):
        """If authentication succeeds, redirect to the success URL."""
        user = auth.models.User.objects.create_user('asdf', 'test@example.com')

        request = self.factory.post('/browserid/verify', {'assertion': 'asdf'})
        with self.settings(LOGIN_REDIRECT_URL='/success'):
            with patch('django_browserid.views.auth.login') as login:
                verify = views.Verify.as_view()
                response = verify(request)

        login.assert_called_with(request, user)
        eq_(response.status_code, 200)
        self.assert_json_equals(response.content,
                                {'email': 'test@example.com', 'redirect': '/success'})

    def test_sanity_checks(self):
        """Run sanity checks on all incoming requests."""
        with patch('django_browserid.views.sanity_checks') as sanity_checks:
            self.verify('post')
        ok_(sanity_checks.called)

    @patch('django_browserid.views.auth.login')
    def test_login_success_no_next(self, *args):
        """
        If _get_next returns None, use success_url for the redirect
        parameter.
        """
        view = views.Verify()
        view.request = self.factory.post('/')
        view.user = Mock(email='a@b.com')

        with patch('django_browserid.views._get_next', return_value=None) as _get_next:
            with patch.object(views.Verify, 'success_url', '/?asdf'):
                response = view.login_success()

        self.assert_json_equals(response.content, {'email': 'a@b.com', 'redirect': '/?asdf'})
        _get_next.assert_called_with(view.request)

    @patch('django_browserid.views.auth.login')
    def test_login_success_next(self, *args):
        """
        If _get_next returns a URL, use it for the redirect parameter.
        """
        view = views.Verify()
        view.request = self.factory.post('/')
        view.user = Mock(email='a@b.com')

        with patch('django_browserid.views._get_next', return_value='/?qwer') as _get_next:
            with patch.object(views.Verify, 'success_url', '/?asdf'):
                response = view.login_success()

        self.assert_json_equals(response.content, {'email': 'a@b.com', 'redirect': '/?qwer'})
        _get_next.assert_called_with(view.request)


class LogoutTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

        _get_next_patch = patch('django_browserid.views._get_next')
        self._get_next = _get_next_patch.start()
        self.addCleanup(_get_next_patch.stop)

    def test_redirect(self):
        """Include LOGOUT_REDIRECT_URL in the response."""
        request = self.factory.post('/')
        logout = views.Logout.as_view()
        self._get_next.return_value = None

        with patch.object(views.Logout, 'redirect_url', '/test/foo'):
            with patch('django_browserid.views.auth.logout') as auth_logout:
                response = logout(request)

        auth_logout.assert_called_with(request)
        eq_(response.status_code, 200)
        self.assert_json_equals(response.content, {'redirect': '/test/foo'})

    def test_redirect_next(self):
        """
        If _get_next returns a URL, use it for the redirect parameter.
        """
        request = self.factory.post('/')
        logout = views.Logout.as_view()
        self._get_next.return_value = '/test/bar'

        with patch.object(views.Logout, 'redirect_url', '/test/foo'):
            with patch('django_browserid.views.auth.logout'):
                response = logout(request)

        self.assert_json_equals(response.content, {'redirect': '/test/bar'})


class CsrfTokenTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.view = views.CsrfToken()

    def test_lazy_token_called(self):
        """
        If the csrf_token variable in the RequestContext is a lazy
        callable, make sure it is called during the view.
        """
        global _lazy_csrf_token_called
        _lazy_csrf_token_called = False

        # I'd love to use a Mock here instead, but lazy doesn't behave
        # well with Mocks for some reason.
        def _lazy_csrf_token():
            global _lazy_csrf_token_called
            _lazy_csrf_token_called = True
            return 'asdf'
        csrf_token = lazy(_lazy_csrf_token, six.text_type)()

        request = self.factory.get('/browserid/csrf/')
        with patch('django_browserid.views.RequestContext') as RequestContext:
            RequestContext.return_value = {'csrf_token': csrf_token}
            response = self.view.get(request)

        eq_(response.status_code, 200)
        eq_(response.content, b'asdf')
        ok_(_lazy_csrf_token_called)

    def test_never_cache(self):
        request = self.factory.get('/browserid/csrf/')
        response = self.view.get(request)
        eq_(response['Cache-Control'], 'max-age=0')



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import base64
import hashlib
import logging

from django.conf import settings

try:
    from django.utils.encoding import smart_bytes
except ImportError:
    from django.utils.encoding import smart_str as smart_bytes

from django_browserid.base import get_audience, LocalVerifier, RemoteVerifier
from django_browserid.signals import user_created
from django_browserid.util import import_from_setting

try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User

    def get_user_model(*args, **kwargs):
        return User


logger = logging.getLogger(__name__)


def default_username_algo(email):
    # store the username as a base64 encoded sha1 of the email address
    # this protects against data leakage because usernames are often
    # treated as public identifiers (so we can't use the email address).
    return base64.urlsafe_b64encode(
        hashlib.sha1(smart_bytes(email)).digest()
    ).rstrip(b'=')


class BrowserIDBackend(object):
    supports_anonymous_user = False
    supports_inactive_user = True
    supports_object_permissions = False

    def __init__(self):
        # Store the current user model on creation to avoid issues if settings.AUTH_USER_MODEL
        # changes, which usually only happens during tests.
        self.User = get_user_model()

    def get_verifier(self):
        """
        Create a verifier for verifying assertions. Uses a
        :class:`django_browserid.base.RemoteVerifier` by default.
        """
        return RemoteVerifier()

    def filter_users_by_email(self, email):
        """Return all users matching the specified email."""
        return self.User.objects.filter(email=email)

    def create_user(self, email):
        """Return object for a newly created user account."""
        from django.db import IntegrityError  # Importing at the top causes issues on DB init.

        username_algo = getattr(settings, 'BROWSERID_USERNAME_ALGO', None)
        if username_algo is not None:
            username = username_algo(email)
        else:
            username = default_username_algo(email)

        try:
            return self.User.objects.create_user(username, email)
        except IntegrityError as err:
            # Race condition! Attempt to re-fetch from the database.
            logger.warning('IntegrityError during user creation: {0}'.format(err))

            try:
                return self.User.objects.get(email=email)
            except self.User.DoesNotExist:
                # Whatevs, let's re-raise the error.
                raise err

    def is_valid_email(self, email):
        """Return True if the email address is ok to log in."""
        # This method is basically for your overriding pleasures.
        return True

    def verify(self, assertion=None, audience=None, request=None, **kwargs):
        """
        Verify the given assertion and audience. See ``authenticate``
        for accepted arguments.
        """
        if audience is None and request:
            audience = get_audience(request)

        if audience is None or assertion is None:
            return None

        verifier = self.get_verifier()
        try:
            result = verifier.verify(assertion, audience, **kwargs)
        except Exception as e:
            result = None
            logger.warn('Error while verifying assertion %s with audience %s.', assertion,
                        audience)
            logger.warn(e)

        if not result:
            return None
        else:
            return result.email

    def authenticate(self, assertion=None, audience=None, request=None, **kwargs):
        """
        Authenticate a user by verifying a BrowserID assertion. Defers
        to the verifier returned by
        :func:`BrowserIDBackend.get_verifier` for verification.

        You may either pass the ``request`` parameter to determine the
        audience from the request, or pass the ``audience`` parameter
        explicitly.

        :param assertion:
            Assertion submitted by the user. This asserts that the user
            controls a specific email address.

        :param audience:
            The audience to use when verifying the assertion; this
            prevents another site using an assertion for their site to
            login to yours. This value takes precedence over the
            audience pulled from the request parameter, if given.

        :param request:
            The request that generated this authentication attempt. This
            is used to determine the audience to use during
            verification, using the
            :func:`django_browserid.base.get_audience` function. If the
            audience parameter is also passed, it will be used instead
            of the audience from the request.

        :param kwargs:
            All remaining keyword arguments are passed to the ``verify``
            function on the verifier.
        """
        email = self.verify(assertion, audience, request, **kwargs)
        if not email or not self.is_valid_email(email):
            return None

        # In the rare case that two user accounts have the same email address,
        # log and bail. Randomly selecting one seems really wrong.
        users = self.filter_users_by_email(email=email)
        if len(users) > 1:
            logger.warn('%s users with email address %s.', len(users), email)
            return None
        if len(users) == 1:
            return users[0]

        create_user = getattr(settings, 'BROWSERID_CREATE_USER', True)
        if not create_user:
            logger.debug('Login failed: No user with email %s found, and '
                         'BROWSERID_CREATE_USER is False', email)
            return None
        else:
            if create_user is True:
                create_function = self.create_user
            else:
                # Find the function to call.
                create_function = import_from_setting('BROWSERID_CREATE_USER')

            user = create_function(email)
            user_created.send(create_function, user=user)
            return user

    def get_user(self, user_id):
        try:
            user = self.User.objects.get(pk=user_id)
            return user
        except self.User.DoesNotExist:
            return None


class LocalBrowserIDBackend(BrowserIDBackend):
    """
    BrowserID authentication backend that uses local verification
    instead of remote verification.
    """
    def get_verifier(self):
        return LocalVerifier()


class AutoLoginBackend(BrowserIDBackend):
    """
    Auto-auths everyone as the user with an email matching the
    BROWSERID_AUTOLOGIN_EMAIL setting. If BROWSERID_AUTOLOGIN_ENABLED
    is False (the default value), auth-auth is disabled.

    .. warning:: Use of this backend creates an insecure site. Do not
                 use on a publicly-visible server!
    """
    def verify(self, *args, **kwargs):
        autologin_enabled = getattr(settings, 'BROWSERID_AUTOLOGIN_ENABLED', False)
        email = getattr(settings, 'BROWSERID_AUTOLOGIN_EMAIL', None)
        if autologin_enabled and email:
            return email
        else:
            return None



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import json

from django.test import TestCase as DjangoTestCase
from django.utils.encoding import smart_text
from django.utils.functional import wraps

from mock import patch
from nose.tools import eq_

from django_browserid.auth import BrowserIDBackend
from django_browserid.base import MockVerifier


def fake_create_user(email):
    pass


class mock_browserid(object):
    """
    Mock verification in :class:`django_browserid.auth.BrowserIDBackend`.

    Can be used as a context manager or as a decorator:

    with mock_browserid('a@b.com'):
        django_browserid.verify('random-token')  # = {'status': 'okay',
                                                 #    'email': 'a@b.com',
                                                 #    ...}

    @mock_browserid(None)
    def browserid_test():
        django_browserid.verify('random-token')  # = False
    """
    def __init__(self, email, **kwargs):
        """
        :param email:
            Email to return in the verification result. If None, the verification will fail.

        :param kwargs:
            Keyword arguments are passed on to :class:`django_browserid.base.MockVerifier`, which
            updates the verification result with them.
        """
        self.patcher = patch.object(BrowserIDBackend, 'get_verifier')
        self.return_value = MockVerifier(email, **kwargs)

    def __enter__(self):
        mock = self.patcher.start()
        mock.return_value = self.return_value
        return mock

    def __exit__(self, exc_type, exc_value, traceback):
        self.patcher.stop()

    def __call__(self, func):
        @wraps(func)
        def inner(*args, **kwargs):
            with self:
                return func(*args, **kwargs)
        return inner


class TestCase(DjangoTestCase):
    def assert_json_equals(self, json_str, value):
        return eq_(json.loads(smart_text(json_str)), value)

    def shortDescription(self):
        # Stop nose using the test docstring and instead the test method
        # name.
        pass



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from django.conf import settings
from django.contrib.auth.models import User
from django.db import IntegrityError

from mock import ANY, Mock, patch

from django_browserid.auth import AutoLoginBackend, BrowserIDBackend, default_username_algo
from django_browserid.base import MockVerifier
from django_browserid.tests import mock_browserid, TestCase

try:
    from django.contrib.auth import get_user_model
    from django_browserid.tests.models import CustomUser
except ImportError:
    get_user_model = False


def new_user(email, username=None):
        """Creates a user with the specified email for testing."""
        if username is None:
            username = default_username_algo(email)
        return User.objects.create_user(username, email)


class BrowserIDBackendTests(TestCase):
    def setUp(self):
        self.backend = BrowserIDBackend()
        self.verifier = Mock()
        self.backend.get_verifier = lambda: self.verifier

    def test_verify_failure(self):
        """If verification fails, return None."""
        self.verifier.verify.return_value = False
        self.assertEqual(self.backend.verify('asdf', 'qwer'), None)
        self.verifier.verify.assert_called_with('asdf', 'qwer')

    def test_verify_success(self):
        """
        If verification succeeds, return the email address from the
        verification result.
        """
        self.verifier.verify.return_value = Mock(email='bob@example.com')
        self.assertEqual(self.backend.verify('asdf', 'qwer'), 'bob@example.com')
        self.verifier.verify.assert_called_with('asdf', 'qwer')

    def test_verify_no_audience_request(self):
        """
        If no audience is provided but a request is, retrieve the
        audience from the request using get_audience.
        """
        request = Mock()
        with patch('django_browserid.auth.get_audience') as get_audience:
            self.backend.verify('asdf', request=request)
            get_audience.assert_called_with(request)
            self.verifier.verify.assert_called_with('asdf', get_audience.return_value)

    def test_verify_no_audience_no_assertion_no_service(self):
        """
        If the assertion isn't provided, or the audience and request
        aren't provided, return None.
        """
        self.assertEqual(self.backend.verify(audience='asdf'), None)
        self.assertEqual(self.backend.verify(assertion='asdf'), None)
        with patch('django_browserid.auth.get_audience') as get_audience:
            get_audience.return_value = None
            self.assertEqual(self.backend.verify('asdf', request=Mock()), None)

    def test_verify_kwargs(self):
        """Any extra kwargs should be passed to the verifier."""
        self.backend.verify('asdf', 'asdf', request='blah', foo='bar', baz=1)
        self.verifier.verify.assert_called_with('asdf', 'asdf', foo='bar', baz=1)

    def auth(self, verified_email=None, **kwargs):
        """
        Attempt to authenticate a user with BrowserIDBackend.

        If verified_email is None, verification will fail, otherwise it will
        pass and return the specified email.
        """
        self.backend.verify = Mock(return_value=verified_email)
        return self.backend.authenticate(assertion='asdf', audience='asdf', **kwargs)

    def test_duplicate_emails(self):
        """
        If there are two users with the same email address, return None.
        """
        new_user('a@example.com', 'test1')
        new_user('a@example.com', 'test2')
        self.assertTrue(self.auth('a@example.com') is None)

    def test_auth_success(self):
        """
        If a single user is found with the verified email, return an
        instance of their user object.
        """
        user = new_user('a@example.com')
        self.assertEqual(self.auth('a@example.com'), user)

    @patch.object(settings, 'BROWSERID_CREATE_USER', False)
    def test_no_create_user(self):
        """
        If user creation is disabled and no user is found, return None.
        """
        self.assertTrue(self.auth('a@example.com') is None)

    @patch.object(settings, 'BROWSERID_CREATE_USER', True)
    def test_create_user(self):
        """
        If user creation is enabled and no user is found, return a new
        User.
        """
        user = self.auth('a@example.com')
        self.assertTrue(user is not None)
        self.assertTrue(isinstance(user, User))
        self.assertEqual(user.email, 'a@example.com')

    @patch.object(settings, 'BROWSERID_CREATE_USER',
                  'django_browserid.tests.test_auth.new_user')
    @patch('django_browserid.tests.test_auth.new_user')
    def test_custom_create_user(self, create_user):
        """
        If user creation is enabled with a custom create function and no
        user is found, return the new user created with the custom
        function.
        """
        create_user.return_value = 'test'
        self.assertEqual(self.auth('a@example.com'), 'test')
        create_user.assert_called_with('a@example.com')

    @patch.object(settings, 'BROWSERID_USERNAME_ALGO')
    @patch.object(settings, 'BROWSERID_CREATE_USER', True)
    def test_custom_username_algorithm(self, username_algo):
        """If a custom username algorithm is specified, use it!"""
        username_algo.return_value = 'test'
        user = self.auth('a@b.com')
        self.assertEqual(user.username, 'test')

    @patch('django_browserid.auth.user_created')
    @patch.object(settings, 'BROWSERID_CREATE_USER', True)
    def test_user_created_signal(self, user_created):
        """
        Test that the user_created signal is called when a new user is
        created.
        """
        user = self.auth('a@b.com')
        user_created.send.assert_called_with(ANY, user=user)

    def test_get_user(self):
        """
        Check if user returned by BrowserIDBackend.get_user is correct.
        """
        user = new_user('a@example.com')
        backend = BrowserIDBackend()
        self.assertEqual(backend.get_user(user.pk), user)

    def test_overriding_valid_email(self):
        class PickyBackend(BrowserIDBackend):
            def is_valid_email(self, email):
                return email != 'a@example.com'

        new_user('a@example.com', 'test1')
        new_user('b@example.com', 'test2')

        with mock_browserid('a@example.com'):
            backend = PickyBackend()
            result = backend.authenticate(assertion='asdf', audience='asdf')
            self.assertTrue(not result)

        with mock_browserid('b@example.com'):
            backend = PickyBackend()
            result = backend.authenticate(assertion='asdf', audience='asdf')
            self.assertTrue(result)

    @patch('django_browserid.auth.logger')
    def test_create_user_integrity_error(self, logger):
        """
        If an IntegrityError is raised during user creation, attempt to
        re-fetch the user in case the user was created since we checked
        for the existing account.
        """
        backend = BrowserIDBackend()
        backend.User = Mock()
        error = IntegrityError()
        backend.User.objects.create_user.side_effect = error
        backend.User.objects.get.return_value = 'asdf'

        self.assertEqual(backend.create_user('a@example.com'), 'asdf')

        # If get raises a DoesNotExist exception, re-raise the original exception.
        backend.User.DoesNotExist = Exception
        backend.User.objects.get.side_effect = backend.User.DoesNotExist
        with self.assertRaises(IntegrityError) as e:
            backend.create_user('a@example.com')
        self.assertEqual(e.exception, error)

    def test_authenticate_verify_exception(self):
        """
        If the verifier raises an exception, log it as a warning and
        return None.
        """
        backend = BrowserIDBackend()
        verifier = Mock()
        exception = Exception()

        backend.get_verifier = lambda: verifier
        verifier.verify.side_effect = exception

        with patch('django_browserid.auth.logger') as logger:
            self.assertEqual(backend.authenticate('asdf', 'asdf'), None)
            logger.warn.assert_called_with(exception)



if get_user_model:
    # Only run custom user model tests if we're using a version of Django that
    # supports it.
    @patch.object(settings, 'AUTH_USER_MODEL', 'tests.CustomUser')
    class CustomUserModelTests(TestCase):
        def _auth(self, backend=None, verified_email=None):
            if backend is None:
                backend = BrowserIDBackend()

            with mock_browserid(verified_email):
                return backend.authenticate(assertion='asdf', audience='asdf')

        def test_existing_user(self):
            """If a custom user exists with the given email, return them."""
            user = CustomUser.objects.create(email='a@test.com')
            authed_user = self._auth(verified_email='a@test.com')
            self.assertEqual(user, authed_user)

        @patch.object(settings, 'BROWSERID_CREATE_USER', True)
        def test_create_new_user(self):
            """
            If a custom user does not exist with the given email, create a new
            user and return them.
            """
            class CustomUserBrowserIDBackend(BrowserIDBackend):
                def create_user(self, email):
                    return CustomUser.objects.create(email=email)
            user = self._auth(backend=CustomUserBrowserIDBackend(),
                              verified_email='b@test.com')
            self.assertTrue(isinstance(user, CustomUser))
            self.assertEqual(user.email, 'b@test.com')


class AutoLoginBackendTests(TestCase):
    def setUp(self):
        self.backend = AutoLoginBackend()

    def test_verify_with_email(self):
        """
        If BROWSERID_AUTOLOGIN_EMAIL is set, use it to auth the user.
        """
        with self.settings(BROWSERID_AUTOLOGIN_EMAIL='bob@example.com',
                           BROWSERID_AUTOLOGIN_ENABLED=True):
            self.assertEqual(self.backend.verify(), 'bob@example.com')

    def test_verify_without_email(self):
        """
        If BROWSERID_AUTOLOGIN_EMAIL is not set, do not auth the user.
        """
        with self.settings(BROWSERID_AUTOLOGIN_EMAIL='', BROWSERID_AUTOLOGIN_ENABLED=True):
            del settings.BROWSERID_AUTOLOGIN_EMAIL
            self.assertEqual(self.backend.verify(), None)

    def test_verify_disabled(self):
        """
        If BROWSERID_AUTOLOGIN_ENABLED is False, do not auth the user
        in any case.
        """
        with self.settings(BROWSERID_AUTOLOGIN_EMAIL='', BROWSERID_AUTOLOGIN_ENABLED=False):
            del settings.BROWSERID_AUTOLOGIN_EMAIL
            self.assertEqual(self.backend.verify(), None)

        with self.settings(BROWSERID_AUTOLOGIN_EMAIL='bob@example.com',
                           BROWSERID_AUTOLOGIN_ENABLED=False):
            self.assertEqual(self.backend.verify(), None)



import json

from django.http import HttpResponse

from django_browserid.util import LazyEncoder


class JSONResponse(HttpResponse):
    """
    HttpResponse that takes in a list or dict and outputs JSON for the response
    body.
    """
    def __init__(self, data, status=200):
        """
        :param data:
            List or dict to serialize to JSON in the response body.

        :param status:
            HTTP status code to use for this response. Defaults to 200.
        """
        data_json = json.dumps(data, cls=LazyEncoder)
        super(JSONResponse, self).__init__(
            data_json, content_type='application/json', status=status)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from django.db import models

try:
    from django.contrib.auth.models import AbstractBaseUser
except ImportError:
    AbstractBaseUser = object


class CustomUser(AbstractBaseUser):
    USERNAME_FIELD = 'email'

    email = models.EmailField(unique=True, db_index=True)

    def get_full_name(self):
        return self.email

    def get_short_name(self):
        return self.email



#!/usr/bin/env python
import codecs
import os
import re
from setuptools import setup, find_packages

# Hack to prevent stupid TypeError: 'NoneType' object is not callable error on
# exit of python setup.py test # in multiprocessing/util.py _exit_function when
# running python setup.py test (see
# http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing
except ImportError:
    pass


def read(*parts):
    return codecs.open(os.path.join(os.path.dirname(__file__), *parts), encoding='utf8').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='django-browserid',
    description='Django application for adding BrowserID support.',
    long_description=read('README.rst'),
    version=find_version('django_browserid/__init__.py'),
    packages=find_packages(),
    author='Paul Osman, Michael Kelly',
    author_email='mkelly@mozilla.com',
    url='https://github.com/mozilla/django-browserid',
    license='MPL v2.0',
    install_requires=['requests>=1.0.0', 'fancy_tag==0.2.0'],
    test_suite="runtests.runtests",
    include_package_data=True,
    zip_safe=False,  # because we rely on finding templates on the filesystem
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
"""Imports that vary depending on available libraries or python versions."""


# If funfactory is available, we want to use its locale-aware reverse
# instead of Django's reverse, so we try to import funfactory's first
# and fallback to Django's if it is not found.
try:
    from funfactory.urlresolvers import reverse
except ImportError:
    from django.core.urlresolvers import reverse


# Use no-op shims for registering template helpers for jingo if it isn't
# found.
class JingoRegister(object):
    def filter(self, func, *args, **kwargs):
        return func

    def function(self, func, *args, **kwargs):
        return func


try:
    from jingo import register as jingo_register
except ImportError:
    jingo_register = JingoRegister()


# If PyBrowserID is installed, we can support local verification.
try:
    import browserid
    pybrowserid_found = True
except ImportError:
    pybrowserid_found = False
